from tkinter import *
import math
#
# ■相対描画のためのRelativeDrawクラス定義
#　  
#   変数名 = RelativeDraw(canvas, startX, startY,scaleX, scaleY, lineWidth,color)
#
#       canvas          : キャンバス
#       startX, startY  : 表示開始位置
#       scaleX, scaleY  : 表示スケール(上下または左右を逆転するときは負の値で指定)
#       lineWidth       : 描画する直線の太さ
#       color           : カラーを文字列で指定(例 : 'red', '#00FFFF')
#
class RelativeDraw:
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        self.canvas=canvas
        self.X0=startX
        self.Y0=startY
        self.SX=scaleX
        self.SY=scaleY
        self.lineWidth=lineWidth
        self.color=color
        self.goto(0,0)
        
    #■ XY座標設定
    def goto(self,x,y):
        self.x=x
        self.y=y
        
    #■ スクリーン座標への変換
    def screenX(self, X):
        return self.X0 + self.SX *  X
    def screenY(self, Y):
        return self.Y0 + self.SY *  Y

class TurtleGraph(RelativeDraw):
    #■ コンストラクタ
    def __init__(self, canvas,startX, startY,scaleX, scaleY, lineWidth,color):
        RelativeDraw.__init__(self,canvas,startX, startY,scaleX, scaleY,
                              lineWidth,color)
        self.setAng(0)
    #■ 角度設定
    def setAng(self,th):
        self.th=th

    #■ 向きの変更(左回転は負の値で指定)
    def turn(self,th):
        self.th=self.th+th

    #■ 前に進む(後退は負の値で指定)
    def forward(self,L):
        TH=self.th*math.pi/180
        X=self.x+L*math.cos(TH)
        Y=self.y+L*math.sin(TH)
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(X)     , self.screenY(Y)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=X
        self.y=Y

        
#■相対位置まで曲線を描くクラス
class RelativeCurve(RelativeDraw):
    def draw(self,X,Y):
        NX=self.x+X
        NY=self.y+Y
        self.canvas.create_line(self.screenX(self.x), self.screenY(self.y),
                                self.screenX(NX)     , self.screenY(NY)     ,
                                width = self.lineWidth, fill = self.color)
        self.x=NX
        self.y=NY
        
#■ヒルベルト曲線のクラス定義(主処理以外はRelativeDrawクラスからの継承)
class HilbertCurve(RelativeCurve):
    #移動量の設定
    def setH(self,order):
        self.ID=order
        H = 1
        S = 2000
        S = S / H
        for i in range(2, order+1):
            H=H*2+1
        self.H=S/H
    #ヒルベルト曲線を描く
    def curve(self, order):
        self.setH(order)
        self.goto(0,0)
        self.RUL()
    #右→上→左
    def RUL(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.URD()
            self.draw(self.H,0)
            self.RUL()
            self.draw(0,self.H)
            self.RUL()
            self.draw(-self.H,0)
            self.DLU()
            self.ID=self.ID+1
    #下→左→上        
    def DLU(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.LDR()
            self.draw(0,-self.H)
            self.DLU()
            self.draw(-self.H,0)
            self.DLU()
            self.draw(0,self.H)
            self.RUL()
            self.ID=self.ID+1
    #左→下→右
    def LDR(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.DLU()
            self.draw(-self.H,0)
            self.LDR()
            self.draw(0,-self.H)
            self.LDR()
            self.draw(self.H,0)
            self.URD()
            self.ID=self.ID+1
    #上→右→下
    def URD(self):
        if self.ID>0:
            self.ID=self.ID-1
            self.RUL()
            self.draw(0,self.H)
            self.URD()
            self.draw(self.H,0)
            self.URD()
            self.draw(0,-self.H)
            self.LDR()
            self.ID=self.ID+1
    
#■ tkinter，Canvas共通処理
def initTk(Title):
    tk=Tk()
    tk.title(Title)
    return tk
def initCanvas(W,H):
    canvas=Canvas(tk,width=600,height=300)
    canvas.pack()
    return canvas

#■　メイン処理
tk=initTk("Hilbert Curve used tkinter")
canvas=initCanvas(600,300)
h=HilbertCurve(canvas, 50, 250, 0.1, -0.1, 2, 'blue')
h.curve(5)
tk.mainloop()
            
        
